function [selected,sparsity,n_iter,beta] = glopridu2_regpath(X,Y,blocks,tau_values,varargin)
% GLOPRIDU2_REGPATH computes the regularization path of GLO_ALGORITHM w. decreasing 
% values of TAU_VALUES, and warm starting.
% 
% [SELECTED] = GLOPRIDU_REGPATH(X,Y,BLOCKS,TAU_VALUES) for each value in TAU_VALUES
%   evaluates grl solution with l2 parameter 0, and builds cell of indexes 
%   of selected features. X and Y are the input matrices and output vectors, respectively, 
%   BLOCKS is either the number of blocks (with equal cardinality) or a cell array, 
%   where element i contains the indexes of the features in block i.
% 
% [SELECTED,SPARSITY] = GLOPRIDU_REGPATH(X,Y,BLOCKS,TAU_VALUES) also returns a vector
%   with the number of selected features for each value in TAU_VALUES
% 
% [SELECTED,SPARSITY,K] = GLOPRIDU_REGPATH(X,Y,BLOCKS,TAU_VALUES) also returns a 
%   vector with the number of kations for each value in TAU_VALUES
% 
% [SELECTED,SPARSITY,K,BETA] = GLOPRIDU_REGPATH(X,Y,TAU_VALUES) also returns 
%  the coefficients vector for each value in TAU_VALUES
% 
% GLOPRIDU_REGPATH(...,'PropertyName',PropertyValue,...) sets properties to the
%   specified property values.
%       -'max_iter_int': (default is 1e4) maximum number of inner iterations
%       -'max_iter_ext': (default is 1e2) maximum number of outer iterations
%       -'tolerance_int': (default is 1e-3) tolerance for stopping the inner iterations.
%       -'tolerance_ext': (default is 1e-6) tolerance for stopping the outer iterations.
%       -'all_path': (default is true) if true evaluates all regularization
%        path with given tolerance. Otherwise evaluates solutions for larger 
%        vaues of tau with looser tolerance
   

   
    if nargin<4; error('too few inputs!'); end
    % DEFAULT PARAMETERS
    max_iter_ext = 1e4;
    max_iter_int = 1e2;
    tol_ext = 1e-6;
    tol_int = 1e-3;
    all_path = true;

    % OPTIONAL PARAMETERS
    args = varargin;
    nargs = length(args);
    for i=1:2:nargs
        switch args{i},
            case 'max_iter_ext'
                max_iter_ext = args{i+1};
            case 'max_iter_int'
                max_iter_int = args{i+1};
            case 'tolerance_int'
                tol_int = args{i+1};
            case 'tolerance_ext'
                tol_ext = args{i+1};
            case 'all_path'
                all_path = args{i+1};
        end
    end
    
    T = length(tau_values);
    [n,d] = size(X);
    
    % if interested in just the first value of TAU_VALUES, evaluates solutions
    % for larger vaues of tau with looser tolerance
    tol_ext = ones(T,1).*tol_ext;
    if ~all_path;
        tol_ext(2:end) = tol_ext(2:end).*100;
    end
    
    sigma = normest(X*X')/n; %step size

    selected = zeros(d,T);
    sparsity = zeros(T,1);
    n_iter = zeros(T,1);
    if nargout>2; beta = cell(T,1); end
    
    beta0 = zeros(d,1);
    sparsity_prev = 0;
    
    lambda0 = [];
    
    stop_par.tol_int = tol_int;
    stop_par.max_iter_ext = max_iter_ext;
    stop_par.max_iter_int = max_iter_int;

    %%%%%%%%%%%%%%%%%%%%%%% REGULARIZATION PATH %%%%%%%%%%%%%%%%%%%%%%%%
    for t = 1:T;
        % if for larger value of tau the algorithm selected less than n 
        % variables (or d if d<n), then keep running the algorithm for 
        % smaller values, o.w. take rls solution
        if sparsity_prev>=min(n,d);
            selected(:,T+1-t) = d;
            if nargout>2;
                if ~exist('beta_ls','var');
                    beta_ls = rls_algorithm(X,Y,0);
                end
                beta_tmp = beta_ls;
            end
        else
            stop_par.tol_ext = tol_ext(T+1-t);
            [beta_tmp,lambda,n_iter(T+1-t)] = glopridu2_algorithm(X,Y,blocks,tau_values(T+1-t),beta0,lambda0,sigma,stop_par);
        end
        lambda0 = lambda; %warm starting
        beta0 = beta_tmp; %warm starting
        selected(:,T+1-t) = beta_tmp~=0;
        sparsity(T+1-t) = sum(selected(:,T+1-t)); % number of selected variables
        sparsity_prev = sum(selected(:,T+1-t));
        if nargout>2;
            beta{T+1-t} = beta_tmp;
        end
    end
%     fprintf('\nregularization path completed\n')
    
    if ~all_path;
        if nargout>2, beta = beta{1}; end
        selected = selected(:,1);
        sparsity = sparsity(1);
    end
end