function [cv_output,model] = glopridu_kcv(X,Y,varargin)
%GLOPRIDU_KCV Parameters choice through cross validation for the glopridu_algorithm 
%(variable selector) followed by Regularized Least Squares (for debiasing).
%   CV_OUTPUT = GLOPRIDU_KCV(X,Y) Given training set, (X,Y), performs leave-one-out cross
%   validation for GLOPRIDU algorithm. GLOPRIDU is used for selection of the variables,
%   whereas the regression coefficient are evaluated on the selected variables 
%   via RLS. The input data X is a NxD matrix, and the labels Y are a Nx1
%   vector
%   [CV_OUTPUT,MODEL] = GLOPRIDU_KCV(X,Y) Also returns the estimated model
%  
%   GLOPRIDU_KCV(...,'PropertyName',PropertyValue,...) sets properties to the
%   specified property values.
%       -'blocks': number of blocks (with equal cardinality) or a cell array,
%        where element i contains the indexes of the features in block i.
%       -'norm_type': only '2' is allowed
%       -'L1_n_par': number of values for the L1 parameter (default is 100) 
%       -'L1_max_par': maximum value for the L1 parameter (default is
%        chosen automatically, see paper)
%       -'L1_min_par': minimum value for the L1 parameter (default is
%        chosen automatically)
%       -'L1_pars': vector of values for the L1 parameter. When not specified,
%        100 values are chosen automatically, or according with
%        'L1_n_par','L1_max_par', and 'L1_min_par'.
%       -'RLS_pars': values for RLS parameter. When not specified,
%        50 values are chosen automatically.
%       -'smooth_par': value of the smoothing parameter (default is 0)
%       -'err_type': 'regr'(deafult) for regression, 'class' for 
%        classfication
%       -'protocol': (default is 'two_steps') if 'one_step' evaluates error
%        of the model learned via GLOPRIDU without RLS-debiasing; if
%        'two_steps' evaluates error of the model learned via GLOPRIDU with 
%        RLS-debiasing; if 'both' evaluates both.
%       -'offset': (default is true) add unpenalized offset to the model.
%       -'K': specify number K of folds in in  K-fold cross-validation.
%        If K=0 or K=length(Y) it performs LOO cross-validation
%       -'rand_split': if false (default) perfoms a deterministic split of
%       the data, if true the split is random.
%       -'plot': (default is false) if true plots training,  validation 
%        errors, and number of selected variables vs the L1 parameter.
%
%   CV_OUTPUT's fields
%	-sparsity: number of selected features for each value of the L1 parameter
% 	-selected_all: indexes of the selected features for each value of the
% 	 L1 parameter.
%   if '1step'==true:
%       -tau_opt_1step: L1 parameter minimizing the K-fold
%        cross-validation error for the 1-step framework (group-L1L2 only)
%       -err_KCV_1step: cross-validation error on validation set for the
%        1-step framework
%       -err_train_1step: cross-validation error on training set for the
%        1-step framework
%   if '2steps'==true:
%       -tau_opt_2steps: L1 parameter minimizing the K-fold
%        cross-validation error for the 2-steps framework (group-L1L2 and RLS)
%       -lambda_opt_2steps: RLS parameter minimizing the K-fold
%        cross-validation error for the 2-steps framework
%       -err_KCV_2steps: cross-validation error on validation set for the
%       2-steps framework
%       -err_train_2steps: cross-validation error on training set for the
%       2-steps framework
%
%   CV_OUTPUT's fields
%	-sparsity(array of double): number of selected features for each value of the L1 parameter
% 	-selected_all(cell array): indeces of the selected features for each value of the L1 parameter.
%   if 'protocol'=='one_step':
%       -tau_opt_1step(double): L1 parameter minimizing the K-fold
%        cross-validation error for the 1-step framework (GLOPRIDU only)
%       -err_KCV_1step(array of double): cross-validation error on validation set for for
%        the 1-step framework 
%       -err_train_1step(array of double): cross-validation error on training set for the
%        1-step framework 
%   if 'protocol'=='two_steps':
%       -tau_opt_2steps(double): L1 parameter minimizing the K-fold
%        cross-validation error for the 2-steps framework (GLOPRIDU and RLS)
%       -lambda_opt_2steps(double): RLS parameter minimizing the K-fold
%        cross-validation error for the 2-steps framework
%       -err_KCV_2steps(2d array of double): cross-validation error on validation set for the
%        2-steps framework
%       -err_train_2steps(2d array of double): cross-validation error on training set for the
%        2-steps framework
%  if 'protocol'=='both': has both of the above sets of fields.
%
%   MODEL's fields
%   -offset: offset to be added to the estimated model
%   if 'protocol'=='one_step':
%       -selected_1step: indexes of the selected features for the optimal
%        parameters  for the 1-step framework
%       -beta_1step: coefficient vector for the optimal parameters for the
%        1-step framework
%   if 'protocol'=='two_steps':
%       -selected_2steps: indexes of the selected features for the optimal
%        parameters for the 2-steps framework
%       -beta_2steps: coefficient vector for the optimal parameters  for
%        the 2-steps framework
%  if 'protocol'=='both': has both of the above sets of fields.
% 
%   See also GLOPRIDU_ALGORITHM, GLOPRIDU_REGPATH, RLS_ALGORITHM
%


if nargin<2, error('too few input!'), end

% DEFAULT PARAMETRS
norm_type = '2';
err_type = 'regr';
ntau = 50;
tau_min = [];
tau_max = []; 
tau_values = [];
lambda_values = [];
K = 0;
split = false;
plotting = false;  
center = true;

% OPTIONAL PARAMETERS
args = varargin;
nargs = length(args);
for i=1:2:nargs
    switch args{i},
		case 'blocks'
            blocks = args{i+1};
		case 'norm_type'
            norm_type = args{i+1};
		case 'L1_pars'
            tau_values = args{i+1};
		case 'L1_min_par'
            tau_min = args{i+1};
		case 'L1_max_par'
            tau_max = args{i+1};
		case 'L1_n_par'
            ntau = args{i+1};
 		case 'RLS_pars'
            lambda_values = args{i+1};
		case 'err_type'
            err_type = args{i+1};
		case 'K'
            K = args{i+1};
		case 'rand_split'
            split = args{i+1};
		case 'offset'
            center = args{i+1};
		case 'plot'
            plotting = args{i+1};
		case 'protocol'
            if strcmp(args{i+1},'one_step');
                withRLS = false;
                woRLS = true;
            elseif strcmp(args{i+1},'two_steps');
                withRLS = true;
                woRLS = false;
            elseif strcmp(args{i+1},'both');
                withRLS = true;
                woRLS = true;
            else
                error('Unknown protocol!!!')
            end
    end
end

switch norm_type;
    case '2';
        regpath = @glopridu2_regpath;
end


[Xtmp,Ytmp] = centering(X,Y,center);

if isempty(tau_values);
    if isempty(tau_max);
        tau_max = glopridu_tau_max(Xtmp,Ytmp,blocks);
    end
    if isempty(tau_min);
        tau_min = tau_max/100;
    end
    tau_values = [tau_min tau_min*((tau_max/tau_min)^(1/(ntau-1))).^(1:(ntau-1))]; %geometric series.
else
    ntau = length(tau_values);
end
if isempty(lambda_values);
    sigma = normest(Xtmp*Xtmp');
    lambda_values = sigma*(10.^(-10:.5:0));
end
clear Xtmp Ytmp

sets = splitting(Y,K,split); %splits the training set in K subsets

% initialization
if woRLS
    err_KCV = zeros(length(sets),ntau);
    err_train = zeros(length(sets),ntau);
end
if withRLS
    err_KCV2 = zeros(length(sets),ntau,length(lambda_values));
    err_train2 = zeros(length(sets),ntau,length(lambda_values));
end
selected = cell(length(sets),1);
sparsity = zeros(length(sets),ntau);

for i = 1:length(sets);
    ind = setdiff(1:length(Y),sets{i}); %indexes of training set
    
    % cenetring
    [Xtr,Ytr,meanX,meanY] = centering(X(ind,:),Y(ind),center);
    Xts = X(sets{i},:);
    Yts = Y(sets{i});
    
    % evaluate all betas for all taus concurrently
    [selected{i},sparsity(i,:),k,beta] = regpath(Xtr,Ytr,blocks,tau_values); 
    selected{i} = selected{i}~=0;
    
    if woRLS
         for t = 1:ntau;
            offset = meanY-meanX*beta{t};
            err_KCV(i,t) = linear_test(Xts,Yts,beta{t},err_type,offset);       
            err_train(i,t) = linear_test(Xtr,Ytr,beta{t},err_type,offset);       
         end
    end
    
    if withRLS
        [sparsity_unique,I_tau] = unique(sparsity(i,:),'first');
        err_KCV_tmp = zeros(length(sparsity_unique),length(lambda_values));
        err_train_tmp = zeros(length(sparsity_unique),length(lambda_values));
        for t = 1:length(sparsity_unique);
            beta = rls_regpath(Xtr(:,selected{i}(:,I_tau(t))),Ytr,lambda_values);
            for j = 1:length(lambda_values);
                offset = meanY-meanX(selected{i}(:,t))*beta{j};
                err_KCV_tmp(t,j) = linear_test(Xts(:,selected{i}(:,I_tau(t))),Yts,beta{j},err_type,offset);       
                err_train_tmp(t,j) = linear_test(Xtr(:,selected{i}(:,I_tau(t))),Ytr,beta{j},err_type,offset);       
            end    
        end
        for t = 1:ntau;
            i_tau = find(sparsity_unique==sum(selected{i}(:,t)),1,'first');
            err_KCV2(i,t,:) = err_KCV_tmp(i_tau,:);       
            err_train2(i,t,:) = err_train_tmp(i_tau,:);       
        end
    end    
    
end

cv_output.selected_all = selected;
cv_output.sparsity = mean(sparsity);
clear selected sparsity

if woRLS
    % evaluate avg. error over the splits
    err_KCV = reshape(mean(err_KCV,1),ntau,1);
    err_train = reshape(mean(err_train,1),ntau,1);
    
    % find L1 parameter minimizing the error
    t_opt = find(err_KCV==min(err_KCV),1,'last');
    cv_output.tau_opt_1step = tau_values(t_opt);
    cv_output.err_KCV_1step = err_KCV;
    cv_output.err_train_1step = err_train;
end

if withRLS
    % evaluate avg. error over the splits
    err_KCV2 = reshape(mean(err_KCV2,1),ntau,length(lambda_values));
    err_train2 = reshape(mean(err_train2,1),ntau,length(lambda_values));

    % for each value of the L1 parameter, find rls parameter minimizing the
    % error
    lambda_opt = zeros(ntau,1);
    err_KCV_opt2 = zeros(ntau,1);
    err_train_opt2 = zeros(ntau,1);
    for t = 1:ntau;
        l_opt = find(err_KCV2(t,:)==min(err_KCV2(t,:)),1,'last');
        lambda_opt(t) = lambda_values(l_opt);
        err_KCV_opt2(t) = err_KCV2(t,l_opt);
        err_train_opt2(t) = err_train2(t,l_opt);
    end
    
    % find L1 parameter minimizing the error
    t_opt2 = find(err_KCV_opt2==min(err_KCV_opt2),1,'last');

    cv_output.tau_opt_2steps = tau_values(t_opt2);
    cv_output.lambda_opt_2steps = lambda_opt(t_opt2);
    cv_output.err_KCV_2steps = err_KCV2;
    cv_output.err_train_2steps = err_train2;  
end

if nargout>1;
    [X,Y,meanX,meanY] = centering(X,Y,center);
    if woRLS;
        [selected,out1,out2,beta] = regpath(X,Y,blocks,tau_values(t_opt:end),'all_path',false); 

        model.selected_1step = selected~=0;
        model.beta_1step = beta;
        model.offset_1step = meanY-meanX*beta;
    end

    if withRLS;
        selected = regpath(X,Y,blocks,tau_values(t_opt2:end),'all_path',false)~=0; 
        beta_rls = rls_algorithm(X(:,selected),Y,lambda_opt(t_opt2));    
        model.selected_2steps = selected;
        model.beta_2steps = zeros(length(selected),1);
        model.beta_2steps(model.selected_2steps) = beta_rls;  
        model.offset_2steps = meanY-meanX(selected)*beta_rls;
    end
end

if plotting;
    figure('Name','Group Lasso with overlap')    
    c = 0;
    if woRLS
        c = c+1;
        subplot(withRLS+woRLS+1,1,c)
        semilogx(tau_values,err_train,'bs-','MarkerSize',3,'MarkerFaceColor','b'); hold on;
        semilogx(tau_values,err_KCV,'rs-','MarkerSize',3,'MarkerFaceColor','r'); 
        legend('train','validation');
        xlim = get(gca,'Xlim');
        ylim = get(gca,'Ylim');
        semilogx(xlim,repmat(min(err_train),2,1),'b:')
        semilogx(xlim,repmat(min(err_KCV),2,1),'r:')
        semilogx(repmat(tau_values(t_opt),2),[ylim(1) min(err_KCV)],'r:')
        title('CV error without RLS');
    end
    if withRLS;
        c = c+1;
        subplot(withRLS+woRLS+1,1,c)
        semilogx(tau_values,err_train_opt2,'bs-','MarkerSize',3,'MarkerFaceColor','b'); hold on;
        semilogx(tau_values,err_KCV_opt2,'rs-','MarkerSize',3,'MarkerFaceColor','r'); 
        legend('train','validation');
        xlim = get(gca,'Xlim');
        ylim = get(gca,'Ylim');
        semilogx(xlim,repmat(min(err_train_opt2), 2,1),'b:')
        semilogx(xlim,repmat(min(err_KCV_opt2), 2,1),'r:')
        semilogx(repmat(tau_values(t_opt2),2),[ylim(1) min(err_KCV_opt2)],'r:')
        title('CV error with RLS');
    end
    c = c+1;
    subplot(withRLS+woRLS+1,1,c)
    semilogx(tau_values,cv_output.sparsity,'ks-','MarkerSize',3,'MarkerFaceColor','b'); 
    hold on
    if woRLS
        semilogx([xlim(1) tau_values(t_opt)],repmat(cv_output.sparsity(t_opt),2,1),'r:')
        semilogx(repmat(tau_values(t_opt),2),[ylim(1) cv_output.sparsity(t_opt)],'r:')        
    end
    if withRLS
        semilogx([xlim(1) tau_values(t_opt2)],repmat(cv_output.sparsity(t_opt2),2,1),'r:')
        semilogx(repmat(tau_values(t_opt2),2),[ylim(1) cv_output.sparsity(t_opt2)],'r:')        
    end
    xlabel('\tau');
    title('# of selected variables');
end
